@@ -64,15 +64,22 @@ from masci_tools.io.parsers.hdf5.reader import Transformation, AttribTransformat
 
 
 def dos_recipe_format(group):
+    """
+    Format for denisty of states calculations retrieving the DOS from the given group
+
+    :param group: str of the group the DOS should be taken from
+
+    :returns: dict of the recipe to retrieve a DOS calculation
+    """
 
     if group == 'Local':
-        atom_prefix = 'MT'
+        atom_prefix = 'MT:'
     elif group == 'jDOS':
-        atom_prefix = 'jDOS'
+        atom_prefix = 'jDOS:'
     elif group == 'Orbcomp':
-        atom_prefix = 'ORBCOMP'
-    elif group == 'mcd':
-        atom_prefix = 'MCD'
+        atom_prefix = 'ORB:'
+    elif group == 'MCD':
+        atom_prefix = 'At'
     else:
         raise ValueError(f'Unknown group: {group}')
 
@@ -85,7 +92,7 @@ def dos_recipe_format(group):
                     Transformation(name='get_all_child_datasets', args=(), kwargs={'ignore': 'energyGrid'}),
                     AttribTransformation(name='add_partial_sums',
                                          attrib_name='atoms_groups',
-                                         args=('{atom_prefix}:{{}}'.format(atom_prefix=atom_prefix).format,),
+                                         args=(f'{atom_prefix}{{}}'.format,),
                                          kwargs={'make_set': True}),
                     Transformation(name='multiply_scalar', args=(1.0 / HTR_TO_EV,), kwargs={}),
                     Transformation(
@@ -103,7 +110,7 @@ def dos_recipe_format(group):
             }
         },
         'attributes': {
-            'dos_group': {
+            'group_name': {
                 'h5path': f'/{group}',
                 'transforms': [
                     Transformation(name='get_name', args=(), kwargs={}),
@@ -155,253 +162,35 @@ def dos_recipe_format(group):
 FleurDOS = dos_recipe_format('Local')
 FleurJDOS = dos_recipe_format('jDOS')
 FleurORBCOMP = dos_recipe_format('Orbcomp')
-FleurMCD = dos_recipe_format('mcd')
-
-#Recipe for bandstructures
-FleurBands = {
-    'datasets': {
-        'weights': {
-            'h5path':
-            '/Local/BS',
-            'transforms': [
-                Transformation(name='get_all_child_datasets', args=(), kwargs={'ignore': ['eigenvalues', 'kpts']}),
-                AttribTransformation(name='add_partial_sums',
-                                     attrib_name='atoms_groups',
-                                     args=('MT:{}'.format,),
-                                     kwargs={'make_set': True}),
-                Transformation(name='split_array', args=(), kwargs={'suffixes': ['up', 'down']}),
-                Transformation(name='flatten_array', args=(), kwargs={})
-            ],
-            'unpack_dict':
-            True
-        },
-        'eigenvalues': {
-            'h5path':
-            '/Local/BS/eigenvalues',
-            'transforms': [
-                AttribTransformation(name='shift_by_attribute',
-                                     attrib_name='fermi_energy',
-                                     args=(),
-                                     kwargs={
-                                         'negative': True,
-                                     }),
-                Transformation(name='multiply_scalar', args=(HTR_TO_EV,), kwargs={}),
-                Transformation(name='split_array', args=(), kwargs={
-                    'suffixes': ['up', 'down'],
-                    'name': 'eigenvalues'
-                }),
-                Transformation(name='flatten_array', args=(), kwargs={})
-            ],
-            'unpack_dict':
-            True
-        },
-        'kpath': {
-            'h5path':
-            '/kpts/coordinates',
-            'transforms': [
-                AttribTransformation(name='multiply_by_attribute',
-                                     attrib_name='reciprocal_cell',
-                                     args=(),
-                                     kwargs={'transpose': True}),
-                Transformation(name='calculate_norm', args=(), kwargs={'between_neighbours': True}),
-                Transformation(name='cumulative_sum', args=(), kwargs={}),
-                AttribTransformation(name='repeat_array_by_attribute', attrib_name='nbands', args=(), kwargs={}),
-            ]
-        },
-    },
-    'attributes': {
-        'n_types': {
-            'h5path':
-            '/atoms',
-            'description':
-            'Number of atom types',
-            'transforms': [
-                Transformation(name='get_attribute', args=('nTypes',), kwargs={}),
-                Transformation(name='get_first_element', args=(), kwargs={})
-            ]
-        },
-        'kpoints': {
-            'h5path': '/kpts/coordinates',
-        },
-        'nkpts': {
-            'h5path':
-            '/Local/BS/eigenvalues',
-            'transforms': [
-                Transformation(name='get_shape', args=(), kwargs={}),
-                Transformation(name='index_dataset', args=(1,), kwargs={})
-            ]
-        },
-        'nbands': {
-            'h5path':
-            '/Local/BS/eigenvalues',
-            'transforms': [
-                Transformation(name='get_shape', args=(), kwargs={}),
-                Transformation(name='index_dataset', args=(2,), kwargs={})
-            ]
-        },
-        'atoms_elements': {
-            'h5path': '/atoms/atomicNumbers',
-            'description': 'Atomic numbers',
-            'transforms': [Transformation(name='periodic_elements', args=(), kwargs={})]
-        },
-        'atoms_position': {
-            'h5path': '/atoms/positions',
-            'description': 'Atom coordinates per atom',
-        },
-        'atoms_groups': {
-            'h5path': '/atoms/equivAtomsGroup'
-        },
-        'bravais_matrix': {
-            'h5path': '/cell/bravaisMatrix',
-            'description': 'Coordinate transformation internal to physical for atoms',
-            'transforms': [Transformation(name='multiply_scalar', args=(BOHR_A,), kwargs={})]
-        },
-        'reciprocal_cell': {
-            'h5path': '/cell/reciprocalCell'
-        },
-        'special_kpoint_indices': {
-            'h5path': '/kpts/specialPointIndices',
-            'transforms': [Transformation(name='shift_dataset', args=(-1,), kwargs={})]
-        },
-        'special_kpoint_labels': {
-            'h5path': '/kpts/specialPointLabels',
-            'transforms': [Transformation(name='convert_to_str', args=(), kwargs={})]
-        },
-        'fermi_energy': {
-            'h5path':
-            '/general',
-            'description':
-            'fermi_energy of the system',
-            'transforms': [
-                Transformation(name='get_attribute', args=('lastFermiEnergy',), kwargs={}),
-                Transformation(name='get_first_element', args=(), kwargs={})
-            ]
-        },
-        'spins': {
-            'h5path':
-            '/general',
-            'description':
-            'number of distinct spin directions in the system',
-            'transforms': [
-                Transformation(name='get_attribute', args=('spins',), kwargs={}),
-                Transformation(name='get_first_element', args=(), kwargs={})
-            ]
-        }
-    }
-}
-
-#Recipe for bandstructures without reading in weights
-FleurSimpleBands = {
-    'datasets': {
-        'eigenvalues': {
-            'h5path':
-            '/Local/BS/eigenvalues',
-            'transforms': [
-                AttribTransformation(name='shift_by_attribute',
-                                     attrib_name='fermi_energy',
-                                     args=(),
-                                     kwargs={
-                                         'negative': True,
-                                     }),
-                Transformation(name='multiply_scalar', args=(HTR_TO_EV,), kwargs={}),
-                Transformation(name='split_array', args=(), kwargs={
-                    'suffixes': ['up', 'down'],
-                    'name': 'eigenvalues'
-                }),
-                Transformation(name='flatten_array', args=(), kwargs={})
-            ],
-            'unpack_dict':
-            True
-        },
-        'kpath': {
-            'h5path':
-            '/kpts/coordinates',
-            'transforms': [
-                AttribTransformation(name='multiply_by_attribute',
-                                     attrib_name='reciprocal_cell',
-                                     args=(),
-                                     kwargs={'transpose': True}),
-                Transformation(name='calculate_norm', args=(), kwargs={'between_neighbours': True}),
-                Transformation(name='cumulative_sum', args=(), kwargs={}),
-                AttribTransformation(name='repeat_array_by_attribute', attrib_name='nbands', args=(), kwargs={}),
-            ]
-        },
-    },
-    'attributes': {
-        'kpoints': {
-            'h5path': '/kpts/coordinates',
-        },
-        'nkpts': {
-            'h5path':
-            '/Local/BS/eigenvalues',
-            'transforms': [
-                Transformation(name='get_shape', args=(), kwargs={}),
-                Transformation(name='index_dataset', args=(1,), kwargs={})
-            ]
-        },
-        'nbands': {
-            'h5path':
-            '/Local/BS/eigenvalues',
-            'transforms': [
-                Transformation(name='get_shape', args=(), kwargs={}),
-                Transformation(name='index_dataset', args=(2,), kwargs={})
-            ]
-        },
-        'bravais_matrix': {
-            'h5path': '/cell/bravaisMatrix',
-            'description': 'Coordinate transformation internal to physical for atoms',
-            'transforms': [Transformation(name='multiply_scalar', args=(BOHR_A,), kwargs={})]
-        },
-        'reciprocal_cell': {
-            'h5path': '/cell/reciprocalCell'
-        },
-        'special_kpoint_indices': {
-            'h5path': '/kpts/specialPointIndices',
-            'transforms': [Transformation(name='shift_dataset', args=(-1,), kwargs={})]
-        },
-        'special_kpoint_labels': {
-            'h5path': '/kpts/specialPointLabels',
-            'transforms': [Transformation(name='convert_to_str', args=(), kwargs={})]
-        },
-        'fermi_energy': {
-            'h5path':
-            '/general',
-            'description':
-            'fermi_energy of the system',
-            'transforms': [
-                Transformation(name='get_attribute', args=('lastFermiEnergy',), kwargs={}),
-                Transformation(name='get_first_element', args=(), kwargs={})
-            ]
-        },
-        'spins': {
-            'h5path':
-            '/general',
-            'description':
-            'number of distinct spin directions in the system',
-            'transforms': [
-                Transformation(name='get_attribute', args=('spins',), kwargs={}),
-                Transformation(name='get_first_element', args=(), kwargs={})
-            ]
-        }
-    }
-}
+FleurMCD = dos_recipe_format('MCD')
 
 
-def get_fleur_bands_specific_weights(weight_name):
+def bands_recipe_format(group, simple=False):
     """
-    Recipe for bandstructure calculations only retrieving one
-    additional weight besides the eigenvalues and kpath
+    Format for bandstructure calculations retrieving weights from the given group
 
-    :param weight_name: key or list of keys of the weight(s) to retrieve
+    :param group: str of the group the weights should be taken from
+    :param simple: bool, if True no additional weights are retrieved with the produced recipe
 
-    :returns: dict of the recipe to retrieve a simple bandstructure
-              plus the one specified weight
+    :returns: dict of the recipe to retrieve a bandstructure calculation
     """
+
+    if group == 'Local':
+        atom_prefix = 'MT:'
+    elif group == 'jDOS':
+        atom_prefix = 'jDOS:'
+    elif group == 'Orbcomp':
+        atom_prefix = 'ORB:'
+    elif group == 'MCD':
+        atom_prefix = 'At'
+    else:
+        raise ValueError(f'Unknown group: {group}')
+
     recipe = {
         'datasets': {
             'eigenvalues': {
                 'h5path':
-                '/Local/BS/eigenvalues',
+                f'/{group}/BS/eigenvalues',
                 'transforms': [
                     AttribTransformation(name='shift_by_attribute',
                                          attrib_name='fermi_energy',
@@ -436,15 +225,11 @@ def get_fleur_bands_specific_weights(weight_name):
             },
         },
         'attributes': {
-            'n_types': {
-                'h5path':
-                '/atoms',
-                'description':
-                'Number of atom types',
+            'group_name': {
+                'h5path': f'/{group}',
                 'transforms': [
-                    Transformation(name='get_attribute', args=('nTypes',), kwargs={}),
-                    Transformation(name='get_first_element', args=(), kwargs={})
-                ]
+                    Transformation(name='get_name', args=(), kwargs={}),
+                ],
             },
             'kpoints': {
                 'h5path': '/kpts/coordinates',
@@ -470,6 +255,16 @@ def get_fleur_bands_specific_weights(weight_name):
                 'description': 'Atomic numbers',
                 'transforms': [Transformation(name='periodic_elements', args=(), kwargs={})]
             },
+            'n_types': {
+                'h5path':
+                '/atoms',
+                'description':
+                'Number of atom types',
+                'transforms': [
+                    Transformation(name='get_attribute', args=('nTypes',), kwargs={}),
+                    Transformation(name='get_first_element', args=(), kwargs={})
+                ]
+            },
             'atoms_position': {
                 'h5path': '/atoms/positions',
                 'description': 'Atom coordinates per atom',
@@ -477,14 +272,14 @@ def get_fleur_bands_specific_weights(weight_name):
             'atoms_groups': {
                 'h5path': '/atoms/equivAtomsGroup'
             },
+            'reciprocal_cell': {
+                'h5path': '/cell/reciprocalCell'
+            },
             'bravais_matrix': {
                 'h5path': '/cell/bravaisMatrix',
                 'description': 'Coordinate transformation internal to physical for atoms',
                 'transforms': [Transformation(name='multiply_scalar', args=(BOHR_A,), kwargs={})]
             },
-            'reciprocal_cell': {
-                'h5path': '/cell/reciprocalCell'
-            },
             'special_kpoint_indices': {
                 'h5path': '/kpts/specialPointIndices',
                 'transforms': [Transformation(name='shift_dataset', args=(-1,), kwargs={})]
@@ -516,6 +311,41 @@ def get_fleur_bands_specific_weights(weight_name):
         }
     }
 
+    if simple:
+        return recipe
+
+    recipe['datasets']['weights'] = {
+        'h5path':
+        f'/{group}/BS',
+        'transforms': [
+            Transformation(name='get_all_child_datasets', args=(), kwargs={'ignore': ['eigenvalues', 'kpts']}),
+            AttribTransformation(name='add_partial_sums',
+                                 attrib_name='atoms_groups',
+                                 args=(f'{atom_prefix}{{}}'.format,),
+                                 kwargs={'make_set': True}),
+            Transformation(name='split_array', args=(), kwargs={'suffixes': ['up', 'down']}),
+            Transformation(name='flatten_array', args=(), kwargs={})
+        ],
+        'unpack_dict':
+        True
+    }
+
+    return recipe
+
+
+def get_fleur_bands_specific_weights(weight_name, group='Local'):
+    """
+    Recipe for bandstructure calculations only retrieving one
+    additional weight besides the eigenvalues and kpath
+
+    :param weight_name: key or list of keys of the weight(s) to retrieve
+    :param group: optional str (default Local) name of the group from where to take the weights
+
+    :returns: dict of the recipe to retrieve a simple bandstructure
+              plus the one specified weight
+    """
+    recipe = bands_recipe_format(group, simple=True)
+
     if isinstance(weight_name, str):
         weight_name = [weight_name]
 
@@ -524,7 +354,7 @@ def get_fleur_bands_specific_weights(weight_name):
             #We need to get all orbitals and calculate the atom sum
             recipe['datasets'][name] = {
                 'h5path':
-                '/Local/BS',
+                f'/{group}/BS',
                 'transforms': [
                     Transformation(name='get_all_child_datasets', args=(), kwargs={'contains': name}),
                     Transformation(name='sum_over_dict_entries', args=(), kwargs={'overwrite_dict': True}),
@@ -540,7 +370,7 @@ def get_fleur_bands_specific_weights(weight_name):
         else:
             recipe['datasets'][name] = {
                 'h5path':
-                f'/Local/BS/{name}',
+                f'/{group}/BS/{name}',
                 'transforms': [
                     Transformation(name='split_array', args=(), kwargs={
                         'suffixes': ['up', 'down'],
@@ -553,3 +383,12 @@ def get_fleur_bands_specific_weights(weight_name):
             }
 
     return recipe
+
+
+#Recipe for bandstructures
+FleurBands = bands_recipe_format('Local')
+FleurOrbcompBands = bands_recipe_format('Orbcomp')
+FleurjDOSBands = bands_recipe_format('jDOS')
+FleurMCDBands = bands_recipe_format('MCD')
+#Recipe for bandstructures without reading in weights
+FleurSimpleBands = bands_recipe_format('Local', simple=True)
