@@ -11,9 +11,78 @@
 #                                                                             #
 ###############################################################################
 """This module contains a simple class for set-like chemical elements enumeration."""
-import dataclasses as dc
+import dataclasses as _dc
+import numpy as _np
+import masci_tools.vis.plot_methods as _plot_methods
 
-import masci_tools.vis.plot_methods
+
+@_dc.dataclass(init=True, repr=True, eq=True, order=False, frozen=False)
+class ChemicalElementsPlottingProfile:
+    """"For reusing plotting settings over several plots. Serializable.
+
+    Instances of this class can be used as input to the :py:meth:`~masci_tools.util.chemical_elements.ChemicalElements.plot`
+    method, to simplify reusing non-changing argumments over a series of periodic table plots. The profile can be saved
+    to and loaded from a JSON file for reuse as well.
+
+    :param profile_name: name of the profile
+    :param values_range: list of sortable, unique values. Must contain at least all plotted attribute values.
+    :param missing_value: Value to set for missing elements.
+    :param missing_name: Name to set for missing elements in legend.
+    :param title_prefix: If set, will be preprended to plot title.
+    :param output_prefix: If set, will be prepended to plot output filepath.
+    :param legend_title_prefix: If set, will be prepended to legend title.
+    :param colorby: 'group': Colormap by selected groups, 'attribute': by mendeleev periodic table attribute.
+    :param attribute: Attribute's value displayed below elements. Either PTE attribute, or group values.
+    :param without_attribute: Do not display the attribute values below the elements.
+    :param colormap_name: Name of seaborn or matplotlib colormap.
+    :param missing_color: Hex code of the color to be used for the missing values (#ffffff white, #bfbfbf light gray).
+    :param missing_group_value: If colorby 'group' and instance is flat, this correct missing colorisation.
+    :param colormap: Dictionary {value : hexcolor string}. If not specified, created from value_range and map name.
+    :param size: Tuple (width, height) of the table figure in pixels.
+    :param showfblock: Show the elements from the f block
+    :param long_version: Show the long version of the periodic table with the f block between the s and d blocks.
+    :param with_legend: True: return extra matplotlib figure = legend. plot(...) will render it below the table.
+    """
+    from masci_tools.util.python_util import dataclass_default_field as _field
+
+    profile_name: str = ''
+    values_range: list = _field([])
+    missing_value: object = None
+    missing_name: str = 'Missing'
+    title_prefix: str = None
+    output_prefix: str = None
+    legend_title_prefix: str = None
+    colorby: str = 'group'
+    attribute: str = 'atomic_weight'
+    without_attribute: bool = False
+    colormap_name: str = 'PiYG'
+    missing_color: str = '#404040'  # dark gray
+    missing_group_value: object = 1
+    colormap: dict = _field({})
+    size: tuple = _field(())
+    showfblock: bool = True
+    long_version: bool = False
+    with_legend: bool = False
+    _version: int = 1
+    _cls_name: str = ''
+
+    def __post_init__(self):
+        """Create colormap from values_range, if the former was not specified."""
+        # save class name
+        cls = type(self)
+        self._cls_name = cls.__module__ + '.' + cls.__name__
+
+        # create colormap
+        if not self.colormap and self.values_range:
+            import seaborn as sns
+            import matplotlib.colors
+            self.colormap = {
+                key: matplotlib.colors.rgb2hex(rgb) for key, rgb in zip(
+                    self.values_range, sns.color_palette(palette=self.colormap_name, n_colors=len(self.values_range)))
+            }
+
+    def save(self, filepath):
+        raise NotImplementedError()
 
 
 class ChemicalElements:
@@ -53,7 +122,7 @@ class ChemicalElements:
         :param distinct: True: disallow same element in different groups.
         :param special_elements: dict symbol:atomic_number of special elements (eg {'X':0} for vacuum)
         :param filepath: if not None, init from JSON file. Must have been written with ChemicalElements.to_file().
-        :type filepath: pathlib.Path
+        :type filepath: str or pathlib.Path
 
         >>> from masci_tools.util.chemical_elements import ChemicalElements
         >>> a = ChemicalElements([11, 2, 118, 78])
@@ -70,13 +139,25 @@ class ChemicalElements:
         >>> assert a.union(b) == ChemicalElements(d.flatten())
         >>> assert e.complement() == c
         """
+        # Define all attributes.
+        # Also see public properties defined below.
+        self.distinct = None
+        self._pte = None
+        self._special_elements = None
+        self._special_elements_inv = None
+        self.__elmts = None
+        self.__data = None
+        self.plotting_profile = None
+
+        # Now initialize them.
+
         # property: distinctness
         self.distinct = distinct
 
         # periodic table, all elements; special element definitions not in actual periodic table
         # devnote: mendeleev.elements.get_all_elements() is very expensive (0.2s). hardcoded pte is faster.
         # self._pte = {el.symbol: el.atomic_number for el in mendeleev.elements.get_all_elements()}
-        #yapf: disable
+        # yapf: disable
         self._pte = {'H': 1, 'He': 2, 'Li': 3, 'Be': 4, 'B': 5, 'C': 6, 'N': 7, 'O': 8, 'F': 9, 'Ne': 10, 'Na': 11,
                      'Mg': 12, 'Al': 13, 'Si': 14, 'P': 15, 'S': 16, 'Cl': 17, 'Ar': 18, 'K': 19, 'Ca': 20, 'Sc': 21,
                      'Ti': 22, 'V': 23, 'Cr': 24, 'Mn': 25, 'Fe': 26, 'Co': 27, 'Ni': 28, 'Cu': 29, 'Zn': 30, 'Ga': 31,
@@ -89,7 +170,7 @@ class ChemicalElements:
                      'U': 92, 'Np': 93, 'Pu': 94, 'Am': 95, 'Cm': 96, 'Bk': 97, 'Cf': 98, 'Es': 99, 'Fm': 100,
                      'Md': 101, 'No': 102, 'Lr': 103, 'Rf': 104, 'Db': 105, 'Sg': 106, 'Bh': 107, 'Hs': 108, 'Mt': 109,
                      'Ds': 110, 'Rg': 111, 'Cn': 112, 'Nh': 113, 'Fl': 114, 'Mc': 115, 'Lv': 116, 'Ts': 117, 'Og': 118}
-        #yapf: enable
+        # yapf: enable
         self._pte_inv = {v: k for k, v in self._pte.items()}
 
         self._special_elements = {}
@@ -104,13 +185,23 @@ class ChemicalElements:
             import json
             with open(filepath, 'r') as file:
                 self.__elmts = json.load(file)
+                # in case any group keys were numeric, they will now be string. rectify that.
+                # in case user WANTS them to be string, can always use 'rename' method to turn them back.
+                from masci_tools.util import python_util
+                group_keys = list(self.__elmts.keys())
+                for key in group_keys:
+                    if python_util.is_number(key):
+                        group_elmts = self.__elmts.pop(key)
+                        self.__elmts[python_util.to_number(key)] = group_elmts
+
+            # init data as empty since to_file() doesn't save data.
             self.__data = {group_name: None for group_name in self.__elmts.keys()}
         else:
             # init elmts, convert to [optional: container of:] dict[s] sym:num.
             import copy
             if groups:
                 if not isinstance(groups, list) \
-                        or not all([isinstance(group_name, str) for group_name in groups]):
+                        or not all(isinstance(group_name, str) for group_name in groups):
                     raise ValueError('If groups not None, must be a list of strings.')
                 if len(groups) < 2:
                     raise ValueError('If groups not None, must be more than one group. '
@@ -130,14 +221,14 @@ class ChemicalElements:
                         # not nested
                         self.__elmts = {'': self._chemical_element_list_to_dict(elements)}
                     elif isinstance(elements, dict):
-                        if not all([isinstance(v, list_types) for k, v in elements.items()]):
+                        if not all(isinstance(v, list_types) for (k, v) in elements.items()):
                             # flact dict
                             elements, _ = self._validate(elements)
                             # lazy type checking for sym:num
                             key_type_is_int = isinstance(list(elements.keys())[0], int)
                             elmts = {v: k for k, v in elements.items()} if key_type_is_int else elements
-                            assert all([isinstance(k, str) for k in elmts.keys()])
-                            assert all([isinstance(v, int) for v in elmts.values()])
+                            assert all(isinstance(k, str) for k in elmts.keys())
+                            assert all(isinstance(v, int) for v in elmts.values())
                             self.__elmts = {'': self._sort(elements)}
                         else:
                             # nested dict
@@ -161,8 +252,13 @@ class ChemicalElements:
                     self.__data = {group_name: None for group_name in group_names}
 
     def is_flat(self):
-        """True if 'elmts' is dict of chemical elements, False if a dict of groups of such.
+        """True if 'elmts' attribute is flat dict of chemical elements, False if a dict of groups of such ('nested').
         """
+        # DEVnote: CE considers elmts with one group as flat dict, and with more than one as nested. CE's interface
+        # changes accordingly. kyes() on a flat dict will return element names, and group names on a nested dict.
+        # That is, CE 'hides' the group structure if there is only a single group. CE does not care about the name
+        # of the single group. The initial name for the group of a flat CE is the empty string. It can be renamed,
+        # and the instance will remain flat.
         return (not list(self.__elmts.keys())) or (list(self.__elmts.keys()) == ['']) or (len(self.__elmts.keys()) == 1)
 
     @property
@@ -278,15 +374,27 @@ class ChemicalElements:
         If flat, returns symbol's atomic number, if nested, returns group of elements.
         If flat, also allows inverted input: atomic_number, return symbol.
         """
-        if isinstance(group_name_or_symbol, str):
-            return self.elmts[group_name_or_symbol]
-        elif isinstance(group_name_or_symbol, int):
-            if self.is_flat():
+        if self.is_flat():
+            if isinstance(group_name_or_symbol, int):
                 return self.invert()[group_name_or_symbol]
+            elif isinstance(group_name_or_symbol, str):
+                return self.elmts[group_name_or_symbol]
             else:
-                raise KeyError('Querying [atomic_number] not possible for nested elmts.')
+                raise KeyError(f"Unsupported key/value type '{type(group_name_or_symbol)}'.")
         else:
-            raise KeyError(f"Unsupported key/value type '{type(group_name_or_symbol)}'.")
+            # accept any type as group symbol type
+            return self.elmts[group_name_or_symbol]
+
+        # # old version:
+        # if isinstance(group_name_or_symbol, str):
+        #     return self.elmts[group_name_or_symbol]
+        # elif isinstance(group_name_or_symbol, int):
+        #     if self.is_flat():
+        #         return self.invert()[group_name_or_symbol]
+        #     else:
+        #         raise KeyError('Querying [atomic_number] not possible for nested elmts.')
+        # else:
+        #     raise KeyError(f"Unsupported key/value type '{type(group_name_or_symbol)}'.")
 
     def __validate_distinctness(self, new__elmts):
         import copy
@@ -797,76 +905,397 @@ class ChemicalElements:
         """Save elements to file. 'data' is ignored.
 
         :param filepath: filepath
-        :type filepath: pathlib.Path
+        :type filepath: str or pathlib.Path
         """
         import json
         with open(filepath, 'w') as file:
             file.write(json.dumps(self.__elmts))
 
-    def plot(self,
-             selected_groups: list = None,
-             selection_name: str = '',
-             unselected_name: str = 'Unspecified',
-             output='notebook'):
-        """Plot a periodic table, elements optionally grouped by group colors.
-        Element groups can either be None, that gives the normal periodic table. Or it can be a list of element symbols
-        ('H', 'Ca' and so on), that will print the periodic table with those elements highlighted and the rest greyed
-        out. Or it can be a dict of element groups, where the key serves as group name. Then each group gets its own
-        color in the periodic table, and a color legend is printed.
+    def _get_mendeleev_periodic_table(self):
+        """Get full periodic table pandas dataframe from mendeleev."""
 
-        :param selected_groups: If flat and not specified, use all elements in flat elmts, else if nested, a group subset.
-        :type selected_groups: list of strings
-        :param selection_name: name for the whole element selection
-        :param unselected_name: legend name for group of unselected, greyed out elements
-        :param output: bokeh plotting output. supported: 'notebook'.
-        """
-        from matplotlib import colors
-        import seaborn as sns
-        import bokeh.plotting
-        import mendeleev.plotting
-
-        if output == 'notebook':
-            bokeh.plotting.output_notebook()
-        else:
-            raise NotImplementedError(f'Output for {output} not implemented.')
-
-        data_selection = self.select_groups(selected_groups, include_special_elements=False)
-
-        # first make another group with all ungrouped elements, to draw them greyed out
-        # note: can't use complement() for this, in case only subset of elmts selected
-        all_elements = [el.symbol for el in mendeleev.elements.get_all_elements()]
-        selected_elements = [element for group in list(data_selection.values()) for element in group]
-        unselected_elements = self._chemical_element_list_to_dict(set(all_elements) - set(selected_elements))
-        data_selection[unselected_name] = unselected_elements
-
-        # get full periodic table pandas dataframe from mendeleev
         # DEVNOTE: breaking change in mendeleev v0.7.0: replaced get_table with fetch.fetch_table.
+        import mendeleev
         version = mendeleev.__version__
-        version_info = tuple([int(num) for num in version.split('.')])
+        version_info = tuple(int(num) for num in version.split('.'))
         if version_info < (0, 7, 0):
             pte = mendeleev.get_table('elements')
         else:
             from mendeleev.fetch import fetch_table
             pte = fetch_table('elements')
 
-        # now make a new attribute(column) and label each element according to group
-        for group_name, group in data_selection.items():
-            for symbol in group.keys():
-                pte.loc[pte['symbol'] == symbol, [selection_name]] = group_name
-        # now make another attribute and map the newly created attribute to a respective color
-        group_names = list(data_selection.keys())
-        cmap = {g: colors.rgb2hex(c) for g, c in zip(group_names, sns.color_palette('deep'))}
-        grey = '#bfbfbf'
-        cmap[unselected_name] = grey
+        return pte
 
-        # if there is more than one group, plot a legend
-        if len(data_selection.keys()) > 2:
-            masci_tools.vis.plot_methods.plot_colortable(cmap, selection_name)
+    def list_of_attributes(self):
+        """Get list of available periodic table attributes from mendeleev."""
+        return self._get_mendeleev_periodic_table().columns
 
-        colorby_attribute = f'{selection_name}_color'
-        pte[colorby_attribute] = pte[selection_name].map(cmap)
-        # and plot
-        mendeleev.plotting.periodic_plot(pte, colorby=colorby_attribute, title=selection_name)
+    def save_plotting_profile(self, filepath=None, force_overwrite: bool = False):
+        """Save plotting profile for :py:meth:`~masci_tools.util.chemical_elements.ChemicalElements.plot`.
+
+        This allows to reuse a plotting profile later, to keep consistency between plots.
+
+        Current format of the file is JSON.
+
+        :param filepath: where to save the the profile file.
+        :type filepath: str or pathlib.Path
+        :param force_overwrite: True: overwrite existing config files. Default False.
+        """
+
+        if not self.plotting_profile:
+            print('Warning: no plotting profile set. I will do nothing.')
+            return
+
+        from masci_tools.util.python_util import JSONEncoderTailoredIndent, NoIndent
+        from pathlib import Path
+
+        if not filepath:
+            _filepath = Path.cwd() / 'imp_host_embedding_batches.json'
+        elif isinstance(filepath, str):
+            _filepath = Path(filepath)
+        else:
+            _filepath = filepath
+
+        # persist
+        import json
+        if _filepath.exists() and _filepath.is_file():
+            msg = f"File '{_filepath}' exists. Force overwrite = {force_overwrite}."
+            msg = f'WARNING: {msg}' if force_overwrite else f'INFO: {msg}'
+            print(msg)
+            if not force_overwrite:
+                return
+
+        import copy
+        data = copy.deepcopy(self.plotting_profile)
+        # assert isinstance(data, ChemicalElementsPlottingProfile) # for autocompletion
+        # prevent lists, dicts, tuples to be item-indented in output JSON
+        data.values_range = NoIndent(data.values_range)
+        data.colormap = NoIndent(data.colormap)
+        data.size = NoIndent(data.size)
+        data = _dc.asdict(data)
+
+        with open(_filepath, 'w') as file:
+            file.write(json.dumps(data, cls=JSONEncoderTailoredIndent, indent=4))
+
+    def load_plotting_profile(self, filepath):
+        """Load plotting profile for :py:meth:`~masci_tools.util.chemical_elements.ChemicalElements.plot`.
+
+        :param filepath: where the profile file is saved.
+        :type filepath: str or pathlib.Path
+        """
+        from json import JSONDecodeError
+        import json
+
+        try:
+            with open(filepath, 'r') as file:
+                data = json.load(file)
+        except (FileNotFoundError, JSONDecodeError) as err:
+            print(f'File {filepath} not found or JSON decoding -> dict failed.')
+            raise err
+
+        try:
+            data = ChemicalElementsPlottingProfile(**data)
+            # print(type(data)) # for testing
+            self.plotting_profile = data
+            print(f'Successfully loaded plotting profile from file {filepath}.')
+        except Exception as err:  # errors: Attribute, Assertion, ...
+            dummy = ChemicalElementsPlottingProfile()
+            print(f"Failed loading '{ChemicalElementsPlottingProfile.__name__}' from file "
+                  f'{filepath}. Check if file has required version: {dummy._version}. '
+                  f'If this is satisfied, it is otherwise corrupted.')
+            raise err
+
+    def plot(self,
+             selected_groups: list = None,
+             title: str = '',
+             colorby: str = 'group',
+             attribute: str = None,
+             without_attribute: bool = False,
+             missing_value=None,
+             missing_color: str = '#bfbfbf',
+             missing_name: str = 'Missing',
+             missing_group_value=1,
+             colormap_name: str = 'RdBu_r',
+             colormap: dict = None,
+             size: tuple = (1000, 800),
+             output: str = None,
+             showfblock: bool = True,
+             long_version: bool = False,
+             with_legend: bool = False,
+             legend_title: str = 'Legend',
+             use_plotting_profile: bool = True):
+        """Plot elements in a periodic table, missing elements greyed out.
+
+        The colormap for the present elements can be chosen via the 'colorby' parameter in either of two ways:
+        A) Colorize them by a physical  attribute. For available periodic table attributes, see
+        :py:meth:`~masci_tools.util.chemical_elements.ChemicalElements.list_of_attributes`. B) Colorize them by their
+        group assignment within this instance. If there are no groups (i.e., flat not nested), then all present elements
+        will have one color. If selected groups is None, will include all of the instance's groups.
+
+        The value to display below each element is controlled via the 'attribute' parameter. Apart from
+        physical attributes, group names can also be chosen to be displayed, if they are numeric.
+        To do that, set the same value (argument) for the 'attribute' and 'title' parameters.
+
+        If the title and attribute rendering are not enough, an additional matplotlib figure, acting as legend,
+        can be returned.
+
+        For available colormap names, see:
+
+        - https://seaborn.pydata.org/tutorial/color_palettes.html
+        - https://matplotlib.org/stable/tutorials/colors/colormaps.html
+
+        :param selected_groups: If flat and not specified, use all elements in flat elmts, else if nested, a group subset.
+        :param title: Title to appear above the periodic table.
+        :param colorby: 'group': Colormap by selected groups, 'attribute': by mendeleev periodic table attribute.
+        :param attribute: Attribute's value displayed below elements. Either PTE attribute, or group values.
+        :param without_attribute: Do not display the attribute values below the elements.
+        :param missing_value: Replaces NaN values, e.g. for custom coloring.
+        :type missing_value: str or numeric. Prefer same type as coloring input (group names or attribute).
+        :param missing_color: Hex code of the color to be used for the missing values (#ffffff white, #bfbfbf light gray).
+        :param missing_name: Name to be used for missing values in the legend. If empty, will use missing_value.
+        :param missing_group_value: If colorby 'group' and instance is flat, this correct missing colorisation.
+        :param colormap_name: Name of seaborn or matplotlib colormap.
+        :param colormap: Dictionary {value : hexcolor string}. If not specified, created from colormap_name.
+        :param size: Tuple (width, height) of the table figure in pixels.
+        :param output: Optional output, e.g. 'img/table.html'. If legend, will also save 'img/table_legend.png'.
+        :param showfblock: Show the elements from the f block.
+        :param long_version: Show the long version of the periodic table with the f block between the s and d blocks.
+        :param with_legend: True: return extra matplotlib figure = legend. plot(...) will render it below the table.
+        :param legend_title: If empty, will replace with table title.
+        :param use_plotting_profile: If a profile has been set, prefer overlay profile arguments over the ones here.
+        :return: Legend or None.
+        """
+        import copy
+
+        _title = title
+        _output = output
+        _legend_title = legend_title
+        _missing_value = missing_value
+        _missing_name = missing_name
+        _missing_color = missing_color
+        _missing_group_value = missing_group_value
+        _colorby = colorby
+        _attribute = attribute
+        _without_attribute = without_attribute
+        _colormap_name = colormap_name
+        _colormap = copy.copy(colormap)
+        _size = size
+        _showfblock = showfblock
+        _long_version = long_version
+        _with_legend = with_legend
+
+        if use_plotting_profile:
+            if not self.plotting_profile:
+                print('Warning: no plotting profile set. I will fall back to method arguments.')
+            else:
+                pp = self.plotting_profile
+                assert isinstance(pp, ChemicalElementsPlottingProfile)  # for autocompletion
+                _title = pp.title_prefix + title if (pp.title_prefix) else title
+                _output = pp.output_prefix + output if (output and pp.output_prefix) else output
+                _legend_title = pp.legend_title_prefix + legend_title if (legend_title and
+                                                                          pp.legend_title_prefix) else legend_title
+                _missing_value = pp.missing_value if pp.missing_value is not None else missing_value
+                _missing_name = pp.missing_name if pp.missing_name else missing_name
+                _missing_color = pp.missing_color if pp.missing_color else missing_color
+                _missing_group_value = pp.missing_group_value if pp.missing_group_value else missing_group_value
+                _colorby = pp.colorby if pp.colorby else colorby
+                if pp.attribute:
+                    _attribute = pp.attribute
+                else:
+                    if attribute == title and pp.title_prefix:
+                        _attribute = pp.title_prefix + attribute
+                    else:
+                        _attribute = attribute
+                _without_attribute = pp.without_attribute if pp.without_attribute is not None else without_attribute
+                _colormap_name = pp.colormap_name if pp.colormap_name else colormap_name
+                _colormap = copy.copy(pp.colormap) if pp.colormap else _colormap
+                _size = pp.size if pp.size else size
+                _showfblock = pp.showfblock if pp.showfblock is not None else showfblock
+                _long_version = pp.long_version if pp.long_version is not None else long_version
+                _with_legend = pp.with_legend if pp.with_legend is not None else with_legend
+
+        # validate inputs
+        valid_colorby_values = ['group', 'attribute']
+        valid_attributes = self.list_of_attributes()
+        if _colorby not in valid_colorby_values:
+            raise KeyError(f"Specified argument 'colorby'='{_colorby}', but must be one of {valid_colorby_values}.")
+        if _attribute != _title and _attribute not in valid_attributes:
+            raise KeyError(f"Specified argument 'attribute'='{_attribute}', but must be one of {valid_attributes}, "
+                           f"or equal argument of parameter 'title'.")
+
+        # imports
+        import matplotlib.colors
+        import seaborn as sns
+        import bokeh.plotting
+        import mendeleev.plotting
+        import numbers
+
+        # init output for notebook. if not notebook, this won't have any effect.
+        bokeh.plotting.output_notebook()
+
+        # set up file output
+        _output_mendel = _output
+        _output_ext = '.html'
+        if _output_mendel:
+            msg_suffix = ''
+            if not _output_mendel.endswith(_output_ext):
+                _output_mendel = _output_mendel + _output_ext
+                msg_suffix = f"Info: filepath did not end in extension '{_output_ext}'. Appended it."
+            print(f'Will write HTML table plot to file {_output_mendel}. {msg_suffix}')
+            bokeh.plotting.output_file(filename=_output_mendel)
+
+        # declare inner variables as arguments for mendeleev plot parameters
+        _colorby_mendel, _attribute_mendel = f'{_title}_color', None
+
+        # get instance's elements, nested or flat
+        groups = self.select_groups(selected_groups, include_special_elements=False)
+
+        # correct for a particular ChemicalElements quirk: if the CE instance was created flat and
+        # groups unchanged, it will (at current implementation) have one single group name, the empty string.
+        # If colorby group, this will lead to mendeleev plotting a white table. To prevent that, could either
+        # rename the group in the instance, or better rename it in the extracted dict, leaving the CE instance
+        # invariant.
+        if _colorby == 'group' and (list(self.__elmts.keys()) == ['']):
+            elements = groups.pop('')
+            groups[_missing_group_value] = elements
+
+            # get the periodic table dataframe
+        ptable = self._get_mendeleev_periodic_table()
+
+        # check some conditionals
+        is_missing_value_numeric = isinstance(_missing_value, numbers.Number)
+
+        def _create_column_from_groups():
+            for group_key, group in groups.items():
+                for symbol in group.keys():
+                    ptable.loc[ptable['symbol'] == symbol, [_title]] = group_key
+
+        def _deal_with_missing_values(_missing_value) -> bool:
+            """Deal with missing values.
+
+            :return: True: filled in missing value, False: not.
+            """
+            fill_in = False
+
+            is_numeric = all(isinstance(item, numbers.Number) for item in ptable[_title].to_list())
+
+            if _missing_value is not None:
+                if _missing_value in ptable[_title].to_list():
+                    print(f'Warning: Specified missing value {_missing_value} would overwrite existing value in '
+                          f"column '{_title}'. Will not replace NaN values with it.")
+                elif is_numeric:
+                    fill_in = is_missing_value_numeric
+                else:
+                    fill_in = True
+                    if is_missing_value_numeric:
+                        _missing_value = str(_missing_value)
+
+            if fill_in:
+                ptable[_title] = ptable[_title].fillna(_missing_value)
+            return fill_in, _missing_value
+
+        legend_figure = None
+
+        # make a new column for the data to be displayed
+        if _colorby == 'attribute':
+            _attribute_mendel = _title
+
+            # copy attribute col to new title col
+            ptable[_title] = ptable[_attribute]
+            # deal with missing values
+            # first, replace values of all items not present in groups
+            missing_elements = self.complement(selected_groups=selected_groups)
+            for symbol in missing_elements:
+                ptable.loc[ptable['symbol'] == symbol, [_title]] = _np.NaN
+
+        elif _colorby == 'group':
+            _attribute_mendel = _attribute
+
+            for group_key, group in groups.items():
+                for symbol in group.keys():
+                    ptable.loc[ptable['symbol'] == symbol, [_title]] = group_key
+
+        if _without_attribute:
+            # create a column with empty values and set that as attribute to display for mendeleev.
+            _attribute_mendel = 'empty'
+            ptable['empty'] = ' '
+
+        values = sorted(ptable[_title].dropna().unique())
+        filled_in, _missing_value = _deal_with_missing_values(_missing_value)
+
+        # # let pandas convert values to most sensible types
+        # # Example use case: when group names are integer, legend would display them as floats without this.
+        # ptable[title] = ptable[title].convert_dtypes()
+        # DEVnote: Commented out, cause this also converts NaN into pandas.NA, and mendeleev ploting method
+        # can't deal with the latter. And seems like legend int problem above solved itself without this.
+
+        # for the title column, either use specified colormap, or create custom one from given name
+        if _colormap:
+            # check if colormap can cover item values. must be a superset or true superset.
+            if not all(val in _colormap.keys() for val in values):
+                _standard_colormap_name = 'RdBu_r'
+                print('Warning: Specified colormap does not cover all values to be colorized. '
+                      f"Will fall back to mendeleev standard colormap '{_standard_colormap_name}'."
+                      f'\nSpecified Colormap: {_colormap}'
+                      f'\nSpecified Colormap name: {_colormap_name}'
+                      f'\nValues to be colorized: {values}')
+                _colormap_name = _standard_colormap_name
+        else:
+            # create custom colormpa from gien name
+            _colormap = {
+                key: matplotlib.colors.rgb2hex(rgb)
+                for key, rgb in zip(values, sns.color_palette(palette=_colormap_name, n_colors=len(values)))
+            }
+
+        if filled_in:
+            if _missing_color in _colormap.values():
+                print(f"Warning: Specified missing color value '{_missing_color}' overwrites existing "
+                      f'color value. Better choose another one.')
+            _colormap[_missing_value] = _missing_color
+
+        # add custom colormap column
+        ptable[_colorby_mendel] = ptable[_title].map(_colormap)
+
+        if with_legend:
+            # for the legend color map, we want to use the missing_name instead of the missing_value,
+            # and put it first. So, got to repopultate the custom colormap.
+            if filled_in:
+                _missing_name_inner = _missing_name if _missing_name else _missing_value
+                # prepend missing name to dict
+                _colormap.pop(_missing_value)
+                cmap_copy = copy.copy(_colormap)
+                _colormap = {_missing_name_inner: _missing_color}
+                for key, hexcolor in cmap_copy.items():
+                    _colormap[key] = hexcolor
+
+            # _legend_title = legend_title if legend_title else title
+            _legend_title_inner = _legend_title
+            if not _legend_title_inner or _legend_title_inner == 'Legend':
+                _legend_title_inner = _legend_title_inner + f' (attribute: {_attribute_mendel})'
+            legend_figure = _plot_methods.plot_colortable(colors=_colormap,
+                                                          title=_legend_title_inner,
+                                                          sort_colors=False)
+
+            # save legend figure if specified
+            if _output_mendel:
+                import os
+                import matplotlib.pyplot as plt
+                _output_legend = os.path.splitext(_output_mendel)
+                _output_legend = _output_legend[0] + '_legend'
+                plt.savefig(_output_legend)
+
+        # finally, draw the plot(s)
+        mendeleev.plotting.periodic_plot(df=ptable,
+                                         attribute=_attribute_mendel,
+                                         title=_title,
+                                         width=_size[0],
+                                         height=_size[1],
+                                         missing=_missing_color,
+                                         colorby=_colorby_mendel,
+                                         output=_output_mendel,
+                                         showfblock=showfblock,
+                                         long_version=long_version)
+        return legend_figure
 
     def _sort(self, a_dict, by_key=False):
         """Sorts chemical element dict by atomic number.
@@ -886,11 +1315,11 @@ class ChemicalElements:
         list_types = (list, tuple, set)
         if isinstance(elements, list_types):
             key_is_symbol = None
-            if all([isinstance(sym, str) for sym in elements]):
-                assert all([elem in self._pte for elem in elements])
+            if all(elem in self._pte for elem in elements):
+                assert all(elem in self._pte for elem in elements)
                 key_is_symbol = True
-            elif all([isinstance(num, int) for num in elements]):
-                assert all([elem in self._pte_inv for elem in elements])
+            elif all(isinstance(num, int) for num in elements):
+                assert all(elem in self._pte_inv for elem in elements)
                 key_is_symbol = False
             else:
                 raise TypeError('received a list, but need list of all atom names or numbers')
@@ -900,14 +1329,14 @@ class ChemicalElements:
             # i know this is not the most efficient way to test this
 
             # first, invert dict if needed to simplify cases
-            if any([isinstance(k, int) for k in elements.keys()]):
-                assert all([isinstance(k, int) for k in elements.keys()])
+            if any(isinstance(k, int) for k in elements.keys()):
+                assert all(isinstance(k, int) for k in elements.keys())
                 elements = {v: k for k, v in elements.items()}
 
             # now assume dict is sym:num, else raise error
-            if any([isinstance(k, str) for k in elements.keys()]):
-                assert all([isinstance(k, str) for k in elements.keys()])
-                assert all([isinstance(v, int) for v in elements.values()])
+            if any(isinstance(k, str) for k in elements.keys()):
+                assert all(isinstance(k, str) for k in elements.keys())
+                assert all(isinstance(v, int) for v in elements.values())
                 key_is_symbol = True
                 return elements, key_is_symbol
             else:
@@ -933,7 +1362,7 @@ class ChemicalElements:
             return a_dict
 
 
-@dc.dataclass
+@_dc.dataclass
 class PeriodicTable:
     """Periodic tables for different properties. Properties may be incomplete.
 
